@@ -153,6 +153,18 @@ namespace xcas {
     do_maple_mode=0; Xcas_Style->value("Xcas");
   }
 
+  static void cb_Xcas_Set_Python1(Fl_Menu_*, void*) {
+    do_maple_mode=256; Xcas_Style->value("Python ^==**");
+  }
+
+  static void cb_Xcas_Set_Python2(Fl_Menu_*, void*) {
+    do_maple_mode=512; Xcas_Style->value("Python ^==xor");
+  }
+
+  static void cb_Xcas_Set_Python4(Fl_Menu_*, void*) {
+    do_maple_mode=1024; Xcas_Style->value("MicroPython");
+  }
+
   static void cb_Xcas_Set_Maple(Fl_Menu_*, void*) {
     do_maple_mode=1; Xcas_Style->value("Maple");
   }
@@ -167,6 +179,11 @@ namespace xcas {
 
   Fl_Menu_Item menu_Xcas_Prg_style[] = {
     {gettext("Xcas"), 0,  (Fl_Callback*)cb_Xcas_Set_CPP, 0, 0, 0, 0, 14, 56},
+    {gettext("Python ^==**"), 0,  (Fl_Callback*)cb_Xcas_Set_Python1, 0, 0, 0, 0, 14, 56},
+    {gettext("Python ^==xor"), 0,  (Fl_Callback*)cb_Xcas_Set_Python2, 0, 0, 0, 0, 14, 56},
+#ifdef HAVE_LIBMICROPYTHON
+    {gettext("MicroPython"), 0,  (Fl_Callback*)cb_Xcas_Set_Python4, 0, 0, 0, 0, 14, 56},
+#endif
     {gettext("Maple"), 0,  (Fl_Callback*)cb_Xcas_Set_Maple, 0, 0, 0, 0, 14, 56},
     {gettext("Mupad"), 0,  (Fl_Callback*)cb_Xcas_Set_Mupad, 0, 0, 0, 0, 14, 56},
     {gettext("TI89/92"), 0,  (Fl_Callback*)cb_Xcas_Set_TI, 0, 0, 0, 0, 14, 56},
@@ -180,11 +197,11 @@ namespace xcas {
     // giac::variables_are_files(Xcas_Save_var->value(),contextptr);
     Xcas_Complex_mode->value(false);
     Xcas_Complex_variables->value(false);
-    do_maple_mode=0; Xcas_Style->value("xcas");
+    do_maple_mode=0; Xcas_Style->value("Xcas");
     Xcas_Increasing_power->value(false);
     Xcas_Angle_radian->value(true);
     Xcas_Approx_mode->value(false);
-    Xcas_Autosimplify->value("regroup");
+    Xcas_Autosimplify->value("1");
     do_scientific_format=0; Xcas_Float_style_output->value("standard");
     do_integer_format=10; Xcas_Integer_style_output->value("10"); 
     do_max_recursion_level=100;Xcas_recursion_level->value(do_max_recursion_level);
@@ -217,7 +234,8 @@ namespace xcas {
     // giac::variables_are_files(Xcas_Save_var->value(),contextptr);
     giac::complex_mode(Xcas_Complex_mode->value(),contextptr);
     giac::complex_variables(Xcas_Complex_variables->value(),contextptr);
-    giac::xcas_mode(contextptr)=do_maple_mode;
+    giac::xcas_mode(contextptr)=do_maple_mode &0xff;
+    giac::python_compat(do_maple_mode/256,contextptr);
     giac::increasing_power(Xcas_Increasing_power->value(),contextptr);
     giac::angle_radian(Xcas_Angle_radian->value(),contextptr);
     giac::approx_mode(Xcas_Approx_mode->value(),contextptr);
@@ -333,7 +351,7 @@ namespace xcas {
     o->align(68|FL_ALIGN_INSIDE);
     }
     { Fl_Input* o = Xcas_Autosimplify = new Fl_Input(175, 205, 55, 25, gettext("autosimplify"));
-    o->tooltip(gettext("Command automatically executed after evaluation (auto-simplification) for example nop or regroup or simplify"));
+    o->tooltip(gettext("Command automatically executed after evaluation (auto-simplification), type 0 for nothing, 1 for regroup and 2 for simplify"));
     o->value("regroup");
     }
     { Fl_Check_Button* o = Xcas_Angle_radian = new Fl_Check_Button(240, 75, 100, 25, gettext("radian"));
@@ -477,16 +495,25 @@ or default eval level)"));
     Xcas_Proba_Epsilon->value(giac::proba_epsilon(contextptr));
     Xcas_Complex_mode->value(giac::complex_mode(contextptr));
     Xcas_Complex_variables->value(giac::complex_variables(contextptr));
-    switch(do_maple_mode=giac::xcas_mode(contextptr)){
-    case 0: Xcas_Style->value("xcas"); break;
-    case 1: Xcas_Style->value("maple"); break;
-    case 2: Xcas_Style->value("mupad"); break;
-    case 3: Xcas_Style->value("ti"); break;
+    do_maple_mode=giac::xcas_mode(contextptr);
+    do_maple_mode+=256*giac::python_compat(contextptr);
+    switch(do_maple_mode){
+    case 0: Xcas_Style->value("Xcas"); break;
+    case 256: Xcas_Style->value("Python ^=**"); break;
+    case 512: Xcas_Style->value("Python ^==xor"); break;
+    case 1024: Xcas_Style->value("MicroPython"); break;
+    case 1: Xcas_Style->value("Maple"); break;
+    case 2: Xcas_Style->value("Mupad"); break;
+    case 3: Xcas_Style->value("Ti"); break;
     }
     Xcas_Increasing_power->value(giac::increasing_power(contextptr));
     Xcas_Angle_radian->value(giac::angle_radian(contextptr));
     Xcas_Approx_mode->value(giac::approx_mode(contextptr));
-    Xcas_Autosimplify->value(giac::autosimplify(contextptr).c_str());
+    string s=unlocalize(giac::autosimplify(contextptr));
+    if (s=="'nop'" || s=="Nop") s="0";
+    if (s=="'regroup'" || s=="regroup") s="1";
+    if (s=="'simplify'" || s=="simplify") s="2";
+    Xcas_Autosimplify->value(s.c_str());
     Xcas_sqrt->value(giac::withsqrt(contextptr));
     Xcas_all_trig_sol->value(giac::all_trig_sol(contextptr));
     Xcas_Set_Digits->value(giac::decimal_digits(contextptr));
@@ -586,10 +613,10 @@ or default eval level)"));
     giac::gnuplot_zmax=Xcas_Zmax->value();
     giac::gnuplot_tmin=Xcas_Tmin->value();
     giac::gnuplot_tmax=Xcas_Tmax->value();
-    xcas::Xcas_config.window_xmin=Xcas_WXmin->value();
-    xcas::Xcas_config.window_xmax=Xcas_WXmax->value();
-    xcas::Xcas_config.window_ymin=Xcas_WYmin->value();
-    xcas::Xcas_config.window_ymax=Xcas_WYmax->value();
+    giac::global_window_xmin=xcas::Xcas_config.window_xmin=Xcas_WXmin->value();
+    giac::global_window_xmax=xcas::Xcas_config.window_xmax=Xcas_WXmax->value();
+    giac::global_window_ymin=xcas::Xcas_config.window_ymin=Xcas_WYmin->value();
+    giac::global_window_ymax=xcas::Xcas_config.window_ymax=Xcas_WYmax->value();
     xcas::Xcas_config.ortho=Xcas_orthonormal->value();
     xcas::Xcas_config.autoscale=Xcas_autoscale->value();
     giac::class_minimum=Xcas_Class_min->value();
@@ -826,6 +853,10 @@ or default eval level)"));
     Xcas_Zmax->value(giac::gnuplot_zmax);
     Xcas_Tmin->value(giac::gnuplot_tmin);
     Xcas_Tmax->value(giac::gnuplot_tmax);
+    xcas::Xcas_config.window_xmin=global_window_xmin;
+    xcas::Xcas_config.window_xmax=global_window_xmax;
+    xcas::Xcas_config.window_ymin=global_window_ymin;
+    xcas::Xcas_config.window_ymax=global_window_ymax;
     Xcas_WXmin->value(xcas::Xcas_config.window_xmin);
     Xcas_WXmax->value(xcas::Xcas_config.window_xmax);
     Xcas_WYmin->value(xcas::Xcas_config.window_ymin);
