@@ -113,7 +113,16 @@ namespace giac {
     }
     if (!assume_t_in_ab(t,plus_inf,plus_inf,true,true,contextptr))
       return gensizeerr(contextptr);
-    gen res=_integrate(gen(makevecteur(f*exp(-t*x,contextptr),x,0,plus_inf),_SEQ__VECT),contextptr);
+    int c=calc_mode(contextptr);
+    calc_mode(0,contextptr);
+    gen res=_integrate(makesequence(f*exp(-t*x,contextptr),x),contextptr);
+    calc_mode(c,contextptr);
+    if (lop(res,at_integrate).empty() && lop(res,at_piecewise).empty() && lop(res,at_sign).empty())
+      res=-_limit(makesequence(res,x,0,1),contextptr);
+    else
+      res=undef;
+    if (is_undef(res))
+      res=_integrate(makesequence(f*exp(-t*x,contextptr),x,0,plus_inf),contextptr);
     for (int i=1;i<=n;++i){
       if (is_undef(res))
 	return res;
@@ -222,7 +231,7 @@ namespace giac {
 	continue;
       }
       if (is_strictly_positive(expa,contextptr))
-	*logptr(contextptr) << gettext("Warning, exponential x coeff is positive ") << expa << endl;
+	*logptr(contextptr) << gettext("Warning, exponential x coeff is positive ") << expa << '\n';
       vecteur varx(lvarx(coeff,x));
       int varxs=int(varx.size());
       if (!varxs){ // Dirac function
@@ -507,7 +516,7 @@ namespace giac {
       return res;
     }
     if (solution_generale.type==_VECT)
-      *logptr(contextptr) << gettext("Boundary conditions for parametric curve not implemented") << endl;
+      *logptr(contextptr) << gettext("Boundary conditions for parametric curve not implemented") << '\n';
     // solve boundary conditions
     iterateur jt=v.begin()+1,jtend=v.end();
     for (unsigned ndiff=0;jt!=jtend;++ndiff,++jt){
@@ -597,14 +606,16 @@ namespace giac {
   static gen desolve_with_conditions(const vecteur & v,const gen & x,const gen & y,gen & f,int step_info,GIAC_CONTEXT){
     if (v.empty())
       return gensizeerr(contextptr);
-    int ordre;
+    int ordre; bool num=false;
     vecteur parameters;
-    gen solution_generale(desolve_f(v.front(),x,y,ordre,parameters,f,step_info,contextptr));
-    if (solution_generale.type!=_VECT) 
-      return in_desolve_with_conditions(v,x,y,solution_generale,parameters,f,step_info,contextptr);
+    gen solution_generale(desolve_f(v.front(),x,y,ordre,parameters,f,step_info,num,contextptr));
+    if (solution_generale.type!=_VECT) {
+      gen res= in_desolve_with_conditions(v,x,y,solution_generale,parameters,f,step_info,contextptr);
+      return num?evalf(res,1,contextptr):res;
+    }
     solution_generale.subtype=0; // otherwise desolve([y'=[[1,2],[2,1]]*y+[x,x+1],y(0)=[1,2]]) fails on the Prime (?)
     if (parameters.empty())
-      return solution_generale;
+      return num?evalf(solution_generale,1,contextptr):solution_generale;
     iterateur it=solution_generale._VECTptr->begin(),itend=solution_generale._VECTptr->end();
     vecteur res;
     res.reserve(itend-it);
@@ -618,7 +629,7 @@ namespace giac {
       else
 	res.push_back(tmp);
     }
-    return res;
+    return num?evalf(res,1,contextptr):res;
   }
 
   static gen desolve_with_conditions(const vecteur & v,const gen & x,const gen & y,gen & f,GIAC_CONTEXT){
@@ -734,7 +745,7 @@ namespace giac {
       cl=ilaplace(cl,x,x,contextptr);
       return vecteur(1,ratnormal(cl,contextptr));
     }
-    gen i=integrate_without_lnabs(rdiv(b,a,contextptr),x,contextptr);
+    gen i=integrate_without_lnabs(inv(a,contextptr)*b,x,contextptr);
     i=normal(lnexpand(i,contextptr),contextptr);
     i=exp(i,contextptr);
     if (step_info)
@@ -844,8 +855,10 @@ namespace giac {
 	    }
 	    if (reel && is_zero(im(cst,contextptr)) && lop(part,at_integrate).empty())
 	      part=re(part,contextptr);
-	    //part=recursive_ratnormal(part,contextptr);
-	    part=simplify(part,contextptr);
+	    if (1) // desolve((y''+y=sin(x)) and (y(0)=1) and (y'(0)=2),y)
+	      part=recursive_ratnormal(part,contextptr);
+	    else
+	      part=simplify(part,contextptr);
 	  }
 	  if (step_info)
 	    gprintf("Particuliar solution %gen",makevecteur(part),step_info,contextptr);
@@ -875,6 +888,15 @@ namespace giac {
       gen & b=v[1];
       gen & c=v[2];
       gen & d=cst;
+#if 0
+      if (is_exactly_zero(c)){
+	vecteur v1(makevecteur(a,b,d));
+	if (desolve_linn(x,y,t,1,v1,parameters,result,step_info,contextptr)){
+	  result=_integrate(makesequence(result,x),contextptr);
+	  return true;
+	}
+      }
+#endif
       gen u=-b/a,V=-c/a,w=-d/a,
 	k=simplify(u*u/4-derive(u,x,contextptr)/2+V,contextptr);
       // y''=u*y'+V*y+w  (with u,V,w functions of x)
@@ -1007,7 +1029,7 @@ namespace giac {
 		vecteur v=*sol._VECTptr;
 		reverse(v.begin(),v.end());
 		sol=symb_horner(-v,x);
-		*logptr(contextptr) << "Polynomial solution found " << sol << endl;
+		*logptr(contextptr) << "Polynomial solution found " << sol << '\n';
 		// now solve equation a*y''+b*y'+c*y+d=0 with y=sol*z
 		// a*sol*z''+(2*a*sol'+b*sol)*z'=d
 		gen res=desolve_lin1(a0*sol,2*a0*derive(sol,x,contextptr)+b0*sol,d,x,parameters,step_info,contextptr);
@@ -1054,12 +1076,13 @@ namespace giac {
     return false;
   }
 
-  gen desolve_f(const gen & f_orig,const gen & x_orig,const gen & y_orig,int & ordre,vecteur & parameters,gen & fres,int step_info,GIAC_CONTEXT){
+  gen desolve_f(const gen & f_orig,const gen & x_orig,const gen & y_orig,int & ordre,vecteur & parameters,gen & fres,int step_info,bool & num,GIAC_CONTEXT){
+    num=false;
     // if x_orig.type==_VECT || y_orig.type==_VECT, they should be evaled
     if (x_orig.type!=_VECT && eval(x_orig,1,contextptr)!=x_orig)
-      return gensizeerr("Independant variable assigned. Run purge("+x_orig.print(contextptr)+")\n");
+      return gensizeerr("Independent variable assigned. Run purge("+x_orig.print(contextptr)+")\n");
     if (y_orig.type!=_VECT && eval(y_orig,1,contextptr)!=y_orig)
-      return gensizeerr("Dependant variable assigned. Run purge("+y_orig.print(contextptr)+")\n");
+      return gensizeerr("Dependent variable assigned. Run purge("+y_orig.print(contextptr)+")\n");
     gen x(x_orig);
     if ( (x_orig.type==_VECT) && (x_orig._VECTptr->size()==1) )
       x=x_orig._VECTptr->front();
@@ -1110,6 +1133,9 @@ namespace giac {
     }
 #endif
     f=remove_equal(eval(f,eval_level(contextptr),contextptr));
+    num=has_num_coeff(f);
+    if (num) 
+      f=exact(f,contextptr);
     if (ckmatrix(f)){
       vecteur v = *f._VECTptr;
       for (int i=0;i<v.size();++i){
@@ -1224,7 +1250,7 @@ namespace giac {
 	  vecteur newsol=solve(res-x,*t._IDNTptr,3,contextptr);
 	  if (is_undef(newsol)){
 	    newsol.clear();
-	    *logptr(contextptr) << "Unable to solve implicit equation "<< res-x << "=0 in " << t << endl;
+	    *logptr(contextptr) << "Unable to solve implicit equation "<< res-x << "=0 in " << t << '\n';
 	  }
 #else
 	  vecteur newsol;
@@ -1233,7 +1259,7 @@ namespace giac {
 	  } catch(std::runtime_error & err){
 	    last_evaled_argptr(contextptr)=NULL;
 	    newsol.clear();
-	    *logptr(contextptr) << "Unable to solve implicit equation "<< res-x << "=0 in " << t << endl;
+	    *logptr(contextptr) << "Unable to solve implicit equation "<< res-x << "=0 in " << t << '\n';
 	  }
 #endif
 	  if (newsol.empty())
@@ -1275,7 +1301,7 @@ namespace giac {
 	  vecteur newsol=solve(implicitsol,*y._IDNTptr,3,contextptr);
 	  if (is_undef(newsol)){
 	    newsol.clear();
-	    *logptr(contextptr) << "Unable to solve implicit equation "<< implicitsol << "=0 in " << y << endl;
+	    *logptr(contextptr) << "Unable to solve implicit equation "<< implicitsol << "=0 in " << y << '\n';
 	  }
 #else
 	  vecteur newsol;
@@ -1286,7 +1312,7 @@ namespace giac {
 	  } catch(std::runtime_error & err){
 	    last_evaled_argptr(contextptr)=NULL;
 	    newsol.clear();
-	    *logptr(contextptr) << "Unable to solve implicit equation "<< implicitsol << "=0 in " << y << endl;
+	    *logptr(contextptr) << "Unable to solve implicit equation "<< implicitsol << "=0 in " << y << '\n';
 	  }
 	  calc_mode(cm,contextptr);
 #endif
@@ -1297,7 +1323,7 @@ namespace giac {
 	  if (step_info)
 	    gprintf("Order 1 x-incomplete differential equation",vecteur(0),step_info,contextptr);
 	  if (debug_infolevel)
-	    *logptr(contextptr) << gettext("Incomplete") << endl;
+	    *logptr(contextptr) << gettext("Incomplete") << '\n';
 	  gen pr=integrate_without_lnabs(inv(*it,contextptr),y,contextptr)+parameters.back();
 	  sol=mergevecteur(sol,solve(pr-x,*y._IDNTptr,3,contextptr));
 	  continue;
@@ -1330,7 +1356,7 @@ namespace giac {
 	  if (step_info)
 	    gprintf("Order 1 Homogeneous differential equation",vecteur(0),step_info,contextptr);
 	  if (debug_infolevel)
-	    *logptr(contextptr) << gettext("Homogeneous differential equation") << endl;
+	    *logptr(contextptr) << gettext("Homogeneous differential equation") << '\n';
 	  tmpsto=sto(doubleassume_and(vecteur(2,0),0,1,false,contextptr),x,contextptr);
 	  if (is_undef(tmpsto))
 	    return tmpsto;
@@ -1345,7 +1371,7 @@ namespace giac {
 	  // Try to find t in x=pr
 	  vecteur v=protect_solve(x-pr,*t._IDNTptr,1,contextptr);
 	  if (!v.empty() && !is_undef(v)){
-	    *logptr(contextptr) << "solve(" << pr << "=" << x << "," << t << ") returned " << v << ".\nIf solutions were missed consider paramplot(" << makevecteur(pr,t*pr) << "," << t << ")" << endl;
+	    *logptr(contextptr) << "solve(" << pr << "=" << x << "," << t << ") returned " << v << ".\nIf solutions were missed consider paramplot(" << makevecteur(pr,t*pr) << "," << t << ")" << '\n';
 	    for (unsigned j=0;j<v.size();++j){
 	      sol.push_back(x*v[j]);
 	    }
@@ -1495,7 +1521,7 @@ namespace giac {
 	    continue;
 	  } // end x-incomplete
 	  gen res(string2gen(gettext("Unable to solve differential equation"),false));
-	  res.subtype=1;
+	  res.subtype=-1;
 	  sol.push_back(res);
 	} 
 	ordre=2;
@@ -1528,6 +1554,7 @@ namespace giac {
   // "unary" version
   gen _desolve(const gen & args,GIAC_CONTEXT){
     if ( args.type==_STRNG && args.subtype==-1) return  args;
+    //if (has_num_coeff(args)) return evalf(_desolve(exact(args,contextptr),contextptr),1,contextptr);
     int ordre;
     vecteur parameters;
     if (args.type!=_VECT || args.subtype!=_SEQ__VECT || (!args._VECTptr->empty() && is_equal(args._VECTptr->back()) && args._VECTptr->back()._SYMBptr->feuille[0].type!=_IDNT)){
@@ -1583,7 +1610,7 @@ namespace giac {
     return ggbputinlist(desolve( v[0],v[1],v[2],ordre,parameters,contextptr),contextptr);    
   }
   static const char _desolve_s []="desolve";
-  static define_unary_function_eval (__desolve,&_desolve,_desolve_s);
+  static define_unary_function_eval_quoted (__desolve,&_desolve,_desolve_s);
   define_unary_function_ptr5( at_desolve ,alias_at_desolve,&__desolve,1,true);
 
   static const char _dsolve_s []="dsolve";
@@ -1620,7 +1647,10 @@ namespace giac {
       y=eval(y,1,contextptr);
     int st=step_infolevel(contextptr);
     step_infolevel(0,contextptr);
-    gen res=desolve_f(f_orig,x,y,ordre,parameters,f,st,contextptr);
+    bool num=false;
+    gen res=desolve_f(f_orig,x,y,ordre,parameters,f,st,num,contextptr);
+    if (num)
+      res=evalf(res,1,contextptr);
     step_infolevel(st,contextptr);
     return res;
   }
@@ -1705,7 +1735,7 @@ namespace giac {
       polynome ipnum(dim),ipden(dim);
       partfrac(num,den,vden,pfde_VECT,ipnum,ipden);
       if (!is_zero(ipnum))
-	*logptr(contextptr) << gettext("Warning, z*argument has a non-zero integral part") << endl;
+	*logptr(contextptr) << gettext("Warning, z*argument has a non-zero integral part") << '\n';
       vector< pf<gen> >::iterator it=pfde_VECT.begin();
       vector< pf<gen> >::const_iterator itend=pfde_VECT.end();
       gen a,A,B;
@@ -2074,7 +2104,7 @@ namespace giac {
     gen laur=_series(makesequence(r,x,inf,1,_POLY1__VECT),contextptr);
     bool success=false;
     if (kovacic_iscase1(poles,dinf)) {
-      cerr << "Case 1 of Kovacic algorithm" << endl;
+      //cerr << "Case 1 of Kovacic algorithm" << '\n';
       /* step 1 */
       gen_map alpha_plus,alpha_minus,sqrt_r;
       gen alpha_inf_plus,alpha_inf_minus;
@@ -2175,7 +2205,7 @@ namespace giac {
       }
     }
     if (!success && kovacic_iscase2(poles)) {
-      cerr << "Case 2 of Kovacic algorithm" << endl;
+      //cerr << "Case 2 of Kovacic algorithm" << '\n';
       /* step 1 */
       gen_map E;
       for (const_iterateur it=poles.begin();it!=poles.end();++it) {
@@ -2268,7 +2298,7 @@ namespace giac {
       }
     }
     if (!success && kovacic_iscase3(cpfr,x,poles,dinf,contextptr)) {
-      cerr << "Case 3 of Kovacic algorithm" << endl;
+      //cerr << "Case 3 of Kovacic algorithm" << '\n';
       vector<int> nv=vecteur_2_vector_int(makevecteur(4,6,12));
       for (vector<int>::const_iterator nt=nv.begin();nt!=nv.end();++nt) {
 	int n=*nt;
@@ -2355,7 +2385,7 @@ namespace giac {
 	    for (int i=0;i<=n;++i) {
 	      ac[i]=_collect(pow(S,i)*P[i+1]/_factorial(n-i,contextptr),contextptr);
 	    }
-	    *logptr(contextptr) << "Warning: outputting the algebraic expression for ω" << endl;
+        //*logptr(contextptr) << "Warning: outputting the algebraic expression for ω" << '\n';
 	    ac=strip_gcd(ac,contextptr);
 	    gen omg=pow(w,4)*ac[4]+pow(w,3)*ac[3]+pow(w,2)*ac[2]+w*ac[1]+ac[0];
 	    if (!is_zero(dy_coeff)) {
@@ -2448,7 +2478,7 @@ namespace giac {
     return kovacicsols(r,x,p,contextptr);
   }
   static const char _kovacicsols_s []="kovacicsols";
-  static define_unary_function_eval (__kovacicsols,&_kovacicsols,_kovacicsols_s);
+  static define_unary_function_eval_quoted (__kovacicsols,&_kovacicsols,_kovacicsols_s);
   define_unary_function_ptr5(at_kovacicsols,alias_at_kovacicsols,&__kovacicsols,_QUOTE_ARGUMENTS,true)
 
   /*
