@@ -86,7 +86,7 @@ namespace giac {
 
   // Is the polynomial v irreducible and primitive modulo p?
   // If it is only irreducible, returns 2 and sets vmin to a primitive poly mod p if primitive is true
-  int is_irreducible_primitive(const modpoly & v,const gen & p,modpoly & vmin,bool primitive,GIAC_CONTEXT){
+  int is_irreducible_primitive(const modpoly & v,const gen & p,modpoly & vmin,int primitive,GIAC_CONTEXT){
     vmin=v;
     int m=int(v.size())-1;
     if (m<2)
@@ -110,8 +110,9 @@ namespace giac {
 	return 0;
       }
     }
-    if (!primitive){
-      *logptr(contextptr) << gettext("Warning, minimal polynomial is only irreducible, not necessarily primitive") << '\n';
+    if (primitive!=1){
+      if (primitive==0)
+	*logptr(contextptr) << gettext("Warning, minimal polynomial is only irreducible, not necessarily primitive") << '\n';
       return 3;
     }
     // Primi: must not divide x^[(p^m-1)/d]-1 for any prime divisor d of p^m-1
@@ -214,7 +215,7 @@ namespace giac {
 	  test[j]=giac_rand(contextptr)%p;
       }
       // *logptr(contextptr) << test << '\n';
-      if (is_irreducible_primitive(test,p,test2,primitive,contextptr))
+      if (is_irreducible_primitive(test,p,test2,primitive?1:0,contextptr))
 	return test2;
     }
     *logptr(contextptr) << gettext("Warning, random search for irreducible polynomial did not work, starting exhaustive search") << '\n';
@@ -228,7 +229,7 @@ namespace giac {
 	test[j]=k1%p;
 	k1/=p;
       }
-      if (is_irreducible_primitive(test,p,test2,primitive,contextptr))
+      if (is_irreducible_primitive(test,p,test2,primitive?1:0,contextptr))
 	return test2;
     }
     return vecteur(1,gensizeerr(gettext("No irreducible primitive polynomial found")));
@@ -340,7 +341,7 @@ namespace giac {
 	  gen P=unmod(_e2r(makesequence(v[1],v[2]),contextptr));
 	  if (P.type==_VECT){
 	    vecteur vmin; // not used
-	    int res=is_irreducible_primitive(*P._VECTptr,v[0],vmin,false,contextptr);
+	    int res=is_irreducible_primitive(*P._VECTptr,v[0],vmin,0,contextptr);
 	    if (res==0)
 	      return gensizeerr("Polynomial "+v[1].print(contextptr)+" is not irreducible modulo "+v[0].print(contextptr));
 	    fieldvalue=galois_field(v[0],smod(P,v[0]),v[2],undef);
@@ -386,7 +387,7 @@ namespace giac {
     if (v[1].type!=_VECT)
       return gensizeerr();
     if (is_undef(v[3])){
-      int res=is_irreducible_primitive(*v[1]._VECTptr,v[0],vmin,primitive,contextptr);
+      int res=is_irreducible_primitive(*v[1]._VECTptr,v[0],vmin,primitive?1:0,contextptr);
       if (!res)
 	return gensizeerr(gettext("Not irreducible or not primitive polynomial")+args.print());
       if (res==2)
@@ -442,22 +443,30 @@ namespace giac {
 
   string galois_field::texprint(GIAC_CONTEXT) const {
     gen xid(x);
+    gen A(a);
+    if (a.type==_INT_){
+      int ai=a.val;
+      if (!ai)
+	return "0";
+      // if (ai==1) return "1";
+      A=char2_uncoerce(a);
+    }
     if (x.type==_VECT && x._VECTptr->size()>=2){
       xid=x._VECTptr->front();
       if (!is_undef(a)){
-	if (x._VECTptr->size()==3 && a.type==_VECT){
-	  if (a._VECTptr->size()==1)
-	    return makemod(a._VECTptr->front(),p).print(contextptr);
-	  gen tmp=symb_horner(*a._VECTptr,x._VECTptr->back());
+	if (x._VECTptr->size()==3 && A.type==_VECT){
+	  if (A._VECTptr->size()==1)
+	    return makemod(A._VECTptr->front(),p).print(contextptr);
+	  gen tmp=symb_horner(*A._VECTptr,x._VECTptr->back());
 	  if (tmp.is_symb_of_sommet(at_plus))
 	    return '('+gen2tex(tmp,contextptr)+')';
 	  else
 	    return gen2tex(tmp,contextptr);
 	}
-	return gen2tex(x._VECTptr->back(),contextptr)+"("+gen2tex(r2e(a,xid,contextptr),contextptr)+")";      
+	return gen2tex(x._VECTptr->back(),contextptr)+"("+gen2tex(r2e(A,xid,contextptr),contextptr)+")";      
       }
     }
-    return string(_galois_field_s)+"("+gen2tex(p,contextptr)+","+gen2tex(r2e(P,xid,contextptr),contextptr)+","+gen2tex(x,contextptr)+","+gen2tex(r2e(a,xid,contextptr),contextptr)+")";
+    return string(_galois_field_s)+"("+gen2tex(p,contextptr)+","+gen2tex(r2e(P,xid,contextptr),contextptr)+","+gen2tex(x,contextptr)+","+gen2tex(r2e(A,xid,contextptr),contextptr)+")";
   }
 
   galois_field::galois_field(const gen & g,bool primitive,GIAC_CONTEXT){
@@ -896,7 +905,7 @@ namespace giac {
   }
 
   // a[astart..aend[ + b[bstart..bend[ to res
-  // M==0 integer addition, M==-1 integer substraction, else gf_char2
+  // M==0 integer addition, M==-1 integer subtraction, else gf_char2
   void gf_char2_addp(const vector<int> & a,int astart,int aend,const vector<int> &b,int bstart,int bend,vector<int> & res,int M){
     res.clear();
     res.reserve(giacmax(aend-astart,bend-bstart));
@@ -1481,7 +1490,7 @@ namespace giac {
       int n=2*(pmin.size()-1);
       int p1=2013265921;
       if (longlong(adeg+bdeg+1)*n<(1<<27)){
-	vector<int> A,B,RES,W,tmpa,tmpb,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8;
+	vector<int> A,B,RES,W,tmpa,tmpb,resp1,resp2,resp3,Wp1,Wp2,Wp3,Wp4,tmp7,tmp8;
 	kronecker_expand(a,n,A);
 	makepositive(&A.front(),A.size(),modulo);
 	kronecker_expand(b,n,B);
@@ -1495,7 +1504,7 @@ namespace giac {
 	vecteur AV,BV,ABV;
 	vector_int2vecteur(A,AV);
 	vector_int2vecteur(B,BV);
-	if (fftmult(AV,BV,modulo-1,modulo-1,ABV,modulo,tmpa,tmpb,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,true)){
+	if (fftmultp1234(AV,BV,modulo-1,modulo-1,ABV,modulo,tmpa,tmpb,resp1,resp2,resp3,Wp1,Wp2,Wp3,Wp4,tmp7,tmp8,true)){
 	  vecteur2vector_int(ABV,modulo,RES);
 	  kronecker_shrink(RES,n,res,pmin,modulo);
 	  return true;
@@ -1929,6 +1938,42 @@ namespace giac {
     return 0;
   }
 
+  int is_irreducible(const vecteur & v,const gen &g){
+    if (v.size()<2) return 0;
+    if (v.size()==2) return 1;
+    gen card;
+    environment env;
+    if (g.type==_MOD){
+      card=*(g._MODptr+1);
+    }
+    else {
+      if (g.type!=_USER)
+	return -1;
+      if (galois_field * gf=dynamic_cast<galois_field *>(g._USERptr)){
+	env.modulo=gf->p;
+	env.coeff=g;
+	card=pow(gf->p,gfsize(gf->P),context0);
+      }
+    }
+    env.modulo=card;
+    env.moduloon=true;
+    gen un(pow(g,0,context0));
+    for (int i=1;i<=(v.size()-1)/2;i++){
+      modpoly xpi(powmod(makevecteur(un,1),pow(card,i,context0)-1,v,&env)),G;
+      if (xpi.empty())
+	xpi.push_back(-un);
+      else {
+	xpi.back()-=un;
+	if (xpi.size()==1 && is_zero(xpi.back()))
+	  xpi.clear();
+      }
+      euclide_gcd(v,xpi,&env,G);
+      if (G.size()>1)
+	return 0;
+    }
+    return 1;
+  }
+
   gen galois_field::makegen(int i) const {
     if (p.type==_ZINT)
       return galois_field(p,P,x,vecteur(1,i));
