@@ -263,6 +263,7 @@
 {"add_language",13,9,0,0},
 {"additionally",13,9,0,0| 1},
 {"adjoint_matrix",13,9,0,0},
+{"affichage",13,9,0,0},
 {"affix",13,9,0,0},
 {"alg",13,9,0,0},
 {"algsubs",13,9,0,0},
@@ -379,6 +380,8 @@
 {"center2interval",13,9,0,0},
 {"centered_cube",13,9,0,0},
 {"centered_tetrahedron",13,9,0,0},
+{"centre",13,9,0,0},
+{"cercle",13,9,0,0| 1},
 {"cfactor",13,9,0,0},
 {"cfsolve",13,9,0,0},
 {"changebase",13,9,0,0},
@@ -441,6 +444,7 @@
 {"convertir",13,9,0,0},
 {"convexhull",13,9,0,0},
 {"coordinates",13,9,0,0},
+{"coordonnees",13,9,0,0},
 {"copy",13,9,0,0},
 {"correlation",13,9,0,0},
 {"cos",13,9,0,0},
@@ -457,6 +461,7 @@
 {"courbe_polaire",13,9,0,0| 1},
 {"covariance",13,9,0,0},
 {"covariance_correlation",13,9,0,0},
+{"cp",13,9,0,0},
 {"cpartfrac",13,9,0,0},
 {"crationalroot",13,9,0,0},
 {"crayon",13,126,0,0},
@@ -510,6 +515,7 @@
 {"diag",13,9,0,0},
 {"diff",13,9,0,0| 1},
 {"dim",13,9,0,0},
+{"dir",13,9,0,0},
 {"display",13,9,0,0},
 {"disque",13,126,0,0},
 {"disque_centre",13,126,0,0},
@@ -534,12 +540,15 @@
 {"down",13,126,0,0},
 {"draw_arc",13,9,0,0},
 {"draw_circle",13,9,0,0},
+{"draw_filled_arc",13,9,0,0},
+{"draw_filled_circle",13,9,0,0},
 {"draw_line",13,9,0,0},
 {"draw_pixel",13,9,0,0},
 {"draw_polygon",13,9,0,0},
 {"draw_rectangle",13,9,0,0},
 {"draw_string",13,9,0,0},
 {"droit",13,9,0,0},
+{"droite",13,9,0,0| 1},
 {"droite_tangente",13,9,0,0},
 {"dsolve",13,9,0,0| 1},
 {"dtype",13,9,0,0},
@@ -643,6 +652,7 @@
 {"float",13,9,0,0},
 {"float2rational",13,9,0,0},
 {"floor",13,9,0,0},
+{"fmod",13,9,0,0},
 {"fonction",13,73,0,0| 1},
 {"fonction_derivee",13,9,0,0},
 {"fopen",13,9,0,0},
@@ -851,10 +861,9 @@
 {"join",13,9,0,0},
 {"jordan",13,9,0,0},
 {"jump",13,126,0,0},
-{"jusque",13,58,0,0| 1},
 {"ker",13,9,0,0},
 {"kernel",13,9,0,0},
-{"keyboard",13,9,0,0},
+{"keydown",13,9,0,0},
 {"kill",13,9,0,0},
 {"kolmogorovd",13,9,0,0},
 {"kolmogorovt",13,9,0,0},
@@ -911,6 +920,7 @@
 {"lname",13,9,0,0},
 {"lncollect",13,9,0,0},
 {"lnexpand",13,9,0,0},
+{"locate",13,9,0,0},
 {"locus",13,9,0,0| 1},
 {"log10",13,9,0,0},
 {"logarithmic_regression",13,9,0,0},
@@ -919,6 +929,7 @@
 {"logistic_regression",13,9,0,0},
 {"logistic_regression_plot",13,9,0,0},
 {"lower",13,9,0,0},
+{"ls",13,9,0,0},
 {"lsmod",13,9,0,0},
 {"lsq",13,9,0,0},
 {"lu",13,9,0,0},
@@ -955,13 +966,16 @@
 {"midpoint",13,9,0,0},
 {"min",13,9,0,0},
 {"min_proba_time",13,9,0,0},
+{"mkdir",13,9,0,0},
 {"mkisom",13,9,0,0},
 {"mksa",13,9,0,0},
 {"modgcd",13,9,0,0},
 {"modgcd_cachesize",13,9,0,0},
 {"modp",13,9,0,0},
 {"mods",13,9,0,0},
+{"monotonic",13,9,0,0},
 {"montre_tortue",13,126,0,0},
+{"more",13,9,0,0},
 {"moustache",13,9,0,0},
 {"moyal",13,9,0,0},
 {"moyenne",13,9,0,0},
@@ -1021,6 +1035,7 @@
 {"ord",13,9,0,0},
 {"order_size",13,9,0,0},
 {"ordinate",13,9,0,0},
+{"ordonnee",13,9,0,0},
 {"orthocenter",13,9,0,0},
 {"orthogonal",13,9,0,0},
 {"os_version",13,9,0,0},
@@ -1052,6 +1067,7 @@
 {"pd",13,126,0,0},
 {"pencolor",13,126,0,0},
 {"pendown",13,126,0,0},
+{"pente",13,9,0,0},
 {"penup",13,126,0,0},
 {"perimeter",13,9,0,0},
 {"perimeterat",13,9,0,0| 1},
@@ -1199,9 +1215,13 @@
 {"rat_jordan_block",13,9,0,0},
 {"rationalroot",13,9,0,0},
 {"ratnormal",13,9,0,0},
+{"rayon",13,9,0,0},
 {"rdiv",13,9,0,0},
 {"re",13,9,0,0},
 {"read",13,77,0,0},
+{"read16",13,9,0,0},
+{"read32",13,9,0,0},
+{"read_nand",13,9,0,0},
 {"readrgb",13,9,0,0},
 {"readwav",13,9,0,0},
 {"real",13,9,0,0},
@@ -1248,6 +1268,7 @@
 {"rhs",13,9,0,0},
 {"right",13,9,0,0},
 {"right_triangle",13,9,0,0| 1},
+{"rm",13,9,0,0},
 {"rm_a_z",13,9,0,0},
 {"rm_all_vars",13,9,0,0},
 {"rmbreakpoint",13,9,0,0| 1},
@@ -1272,6 +1293,7 @@
 {"rpn",13,9,0,0},
 {"rref",13,9,0,0},
 {"rsolve",13,9,0,0| 1},
+{"saisir",13,9,0,0| 1},
 {"saute",13,126,0,0},
 {"sauve",13,126,0,0| 1},
 {"save_history",13,9,0,0},
@@ -1321,6 +1343,7 @@
 {"size",13,9,0,0},
 {"sizes",13,9,0,0},
 {"skip",13,126,0,0},
+{"sleep",13,9,0,0},
 {"slope",13,9,0,0},
 {"slopeat",13,9,0,0| 1},
 {"slopeatraw",13,9,0,0},
@@ -1341,6 +1364,7 @@
 {"sortd",13,9,0,0},
 {"sorted",13,9,0,0},
 {"soundsec",13,9,0,0},
+{"speed",13,126,0,0},
 {"sphere",13,9,0,0},
 {"spherical2rectangular",13,9,0,0},
 {"spline",13,9,0,0},
@@ -1466,6 +1490,7 @@
 {"vertices",13,9,0,0},
 {"vertices_abc",13,9,0,0},
 {"vertices_abca",13,9,0,0},
+{"vitesse_tortue",13,126,0,0},
 {"vpotential",13,9,0,0| 1},
 {"warn_equal_in_prog",13,9,0,0},
 {"watch",13,9,0,0| 1},
@@ -1484,6 +1509,8 @@
 {"wilcoxont",13,9,0,0},
 {"with_sqrt",13,9,0,0},
 {"write",13,9,0,0| 1},
+{"write16",13,9,0,0},
+{"write32",13,9,0,0},
 {"writergb",13,9,0,0},
 {"writewav",13,9,0,0},
 {"wz_certificate",13,9,0,0},
